# -*- coding: utf-8 -*-
import os
from PIL import Image
from io import BytesIO

def getFileSizeReadable(size: int):
    units = ['B', 'KB', 'MB', 'GB', 'TB']
    unitIndex = 0
    while size >= 1024 and unitIndex < len(units) - 1:
        size /= 1024.0
        unitIndex += 1
    return f"{size:.2f} {units[unitIndex]}"

def getImageSize(imageBytes):
    objImage = Image.open(BytesIO(imageBytes))
    return objImage.size

def getImageBytesFormat(imageBytes):
    try:
        with Image.open(BytesIO(imageBytes)) as img:
            return img.format
    except Exception as e:
        pass
    return None

def convertImageQuality(imageBytes, toImageFormat: str = 'jpg', quality: int = 100):
    if not imageBytes:
        return None
    objImage = Image.open(BytesIO(imageBytes))
    imageBuffer = BytesIO()
    toImageFormat = toImageFormat.lower()
    if toImageFormat == 'jpg' or toImageFormat == 'jpeg':
        objImage.convert("RGB").save(imageBuffer, "JPEG", quality=quality)
    elif  toImageFormat == 'png':
        objImage.convert("RGB").save(imageBuffer, "PNG", quality=quality)
    elif  toImageFormat == 'gif':
        objImage.convert("RGB").save(imageBuffer, "GIF", quality=quality)
    return (imageBuffer.getvalue(), objImage.size)

def convertPNGToJPGInMemory(pngBytes, quality: int = 100):
    pngImage = Image.open(BytesIO(pngBytes))
    jpgBuffer = BytesIO()
    pngImage.convert("RGB").save(jpgBuffer, "JPEG", quality=quality)
    jpgBytes = jpgBuffer.getvalue()
    return jpgBytes

def convertPNGFileToJPGFile(outputJPGPath: str, pngPath: str, quality: int = 100):
    pngImage = Image.open(pngPath)
    pngImage.convert("RGB").save(outputJPGPath, "JPEG", quality=quality)

def getImagesFrom(path: str, extension: list[str] = ['.png', '.jpg', '.gif'], sortByFilename: bool = True) -> list[str]:
    output = []
    extension = [ exname.lower() for exname in extension]
    if path and os.path.exists(path):
        for root, dirs, files in os.walk(path):
            for file in files:
                if not extension:
                    output.append((os.path.join(root, file), '', file))
                    continue
                for exname in extension:
                    if file.lower().endswith(exname):
                        output.append((os.path.join(root, file), exname, file))
                        break
        if sortByFilename:
            output.sort()
    return output





# -*- encoding: utf-8 -*-

import os
from ebooklib import epub
import fitz
from PIL import Image
from io import BytesIO
import uuid
import time

from epub_image_helper import common

class EPUBImageHelper:
    def __init__(self, outputEPUBPath: str = '/tmp/test.epub', imageBookId: str = str(uuid.uuid4()), imageBookTitle: str = 'Unknown', imageBookLanguage: str = 'en', imageBookAuthor: list[str] = ['Test'], tableOfContent: list[dict[str, list[str]]] = [], pickFirstImageToBeBookCover: bool = False):
        self.outputEPUBPath = outputEPUBPath
        self.bookId = imageBookId
        self.bookTitle = imageBookTitle
        self.bookLanguage = imageBookLanguage
        self.bookAuthor = imageBookAuthor
        self.hasBookCover = False
        self.pickFirstImageToBeBookCover = pickFirstImageToBeBookCover
        self.bookCoverBytes = None
        self.bookCoverFilename = 'defaultCover.jpg'
        self.tableOfContent = tableOfContent
        self.objectId = 0
        self.imageFormatConversionTable = {}
        self.xhtmlResource = {
            'style': {
                'default.css': '''
html, body { margin: 0; padding: 0; }
                '''
            }
        }
        self.xhtmlTemplate = {
            "image": '''
<svg 
  xmlns="http://www.w3.org/2000/svg"
  version="1.1"
  xmlns:xlink="http://www.w3.org/1999/xlink"
  width="100%" height="100%"
  viewBox="0 0 {imageWidth} {imageHeight}">
  <image
    width="{imageWidth}" height="{imageHeight}" 
    xlink:href="{imagePath}" />
</svg>
''',
        }
    def setTableOfContent(self, inputTOC: list[dict]) -> dict:
        checker = self.veirfyTableOfContent(inputTOC)
        if checker['status']:
            self.tableOfContent = checker['output']
        return checker

    def veirfyTableOfContent(self, inputTOC: list[dict] = []) -> dict:
        output = {"status": False, "error": [], 'input': inputTOC, 'output': [], 'totalImage': 0, 'timeCost': []}
        if inputTOC:
            for index, item in enumerate(inputTOC):
                startTime = time.time()
                for checkField in ['type', 'name']:
                    if not checkField in item:
                        output["error"].append(f'"{checkField}" basic field not found: index={index}, item={item}')
                        continue

                if output["error"]:
                    return output

                if item['type'] not in ['chapter', 'imageDir', 'imagePdf']:
                    output["error"].append(f'"type" value error: index={index}, item={item}')
                    continue

                if item['type'] == 'chapter':
                    output['output'].append(item)

                elif item['type'] == 'imageDir':
                    """
                    {
                        "type": "imageDir",
                        "name": "name",
                        "imageDir": "/tmp/imageDir"
                    }
                    """
                    taskType = item['type']
                    checkFailed = False
                    for checkField in ['imageDir']:
                        if not checkField in item:
                            output["error"].append(f'type="{taskType}" error, "{checkField}" field not found: index={index}, item={item}')
                            checkFailed = False
                    if checkFailed:
                        continue
                    for pathCheck in ['imageDir']:
                        if not os.path.exists(item[pathCheck]):
                            output["error"].append(f'type="{taskType}" error, "{pathCheck}" not exists: index={index}, item={item}')
                            checkFailed = False
                    if checkFailed:
                        continue

                    images = common.getImagesFrom(item['imageDir'])
                    imageCount = len(images)
                    if imageCount == 0:
                        output["error"].append(f'type="imageDir" error, "imageDir" no images: index={index}, item={item}')
                        continue
                    output['totalImage'] += imageCount
                    output['output'].append(item)
                elif item['type'] == 'imagePdf':
                    """
                    {
                        "type": "imagePdf",
                        "name": "name",
                        "imagePdf": "/tmp/test.pdf",
                        "workDir": "/tmp/test.pdf/",
                        "imagePdfSaveFormat": "png",
                        "imagePdfSaveQuality": 100
                    }
                    """
                    taskType = item['type']
                    checkFailed = False
                    for checkField in ['imagePdf', 'workDir', 'imagePdfSaveFormat', 'imagePdfSaveQuality']:
                        if not checkField in item:
                            output["error"].append(f'type="{taskType}" error, "{checkField}" field not found: index={index}, item={item}')
                            checkFailed = True
                    if checkFailed:
                        continue
                    for pathCheck in ['imagePdf', 'workDir']:
                        if not os.path.exists(item[pathCheck]):
                            output["error"].append(f'type="{taskType}" error, "{pathCheck}" not exists: index={index}, item={item}')
                            checkFailed = True
                    if checkFailed:
                        continue
                    item['imagePdfSaveFormat'] = item['imagePdfSaveFormat'].lower()
                    if item['imagePdfSaveFormat'] not in ['jpg', 'png', 'gif']:
                            output["error"].append(f'type="{taskType}" error, "imagePdfSaveFormat" not supported (jpg, png, gif): index={index}, item={item}')
                            checkFailed = True
                    if checkFailed:
                        continue

                    try:
                        pdfDocument = fitz.open(item['imagePdf'])
                        totalImageCount = 0
                        for pageNumber in range(pdfDocument.page_count):
                            page = pdfDocument[pageNumber]
                            images = page.get_images(full=True)
                            for imgIndex, imgInfo in enumerate(images):
                                targetImage = pdfDocument.extract_image(imgInfo[0])
                                saveToPath = os.path.join(item['workDir'], f"{totalImageCount:03}.{item['imagePdfSaveFormat']}")
                                imageBytes, imageSize = common.convertImageQuality(targetImage['image'], item['imagePdfSaveFormat'], item['imagePdfSaveQuality'])
                                with open(saveToPath, "wb") as f:
                                    f.write(targetImage['image'])
                                totalImageCount += 1
                        
                        outputItem = {
                            'type': 'imageDir',
                            'name': item['name'],
                            'imageDir': item['workDir'],
                        }
                        for itemField in ['xhtmlTemplateId', 'autoBuildFirstPage', 'keepEvenNumberedPages']:
                            if itemField in item:
                                outputItem[itemField] = item[itemField]
                        output['output'].append(outputItem)
                        output['totalImage'] += totalImageCount
                    except Exception as e:
                        output["error"].append(f'type="{taskType}" pdf error: {e}')

                output['timeCost'].append(time.time() - startTime)

        output['status'] = len( output["error"]) == 0

        return output

    def setXHTMLStyle(self, cssFilename: str, styleContent: str):
        if 'style' not in self.xhtmlResource:
            self.xhtmlResource['style'] = {}
        self.xhtmlResource['style'][cssFilename] = styleContent

    def disableXHTMLStyle(self, cssFilename: str):
        if 'style' in self.xhtmlResource and cssFilename in self.xhtmlResource['style']:
            del self.xhtmlResource['style'][cssFilename]

    def setXHTMLTemplate(self, target: str, templateContent: str):
        if target and templateContent:
            self.xhtmlTemplate[target] = templateContent

    def enableImageFormatConversion(self, quality: int = 100, fromImageFormat: str = 'png', toImageFormat: str = 'jpg'):
        fromImageFormat = fromImageFormat.lower()
        toImageFormat = toImageFormat.lower()
        self.imageFormatConversionTable[ fromImageFormat ] = { 'to': toImageFormat , 'quality': quality }

    def disableImageFormatConversion(self, fromImageFormat: str):
        fromImageFormat = fromImageFormat.lower()
        if fromImageFormat in self.imageFormatConversionTable:
            del self.imageFormatConversionTable[fromImageFormat]

    def getObjectId(self, prefixName: str = 'id'):
        self.objectId += 1
        return f"{prefixName}-{self.objectId:06}"
 
    def createDefaultCover(self, backgroundColor: str = 'white', width: int = 600, height: int = 800, imageFormat: str = 'png'):
        defaultCover = Image.new('RGB', (width, height), color=backgroundColor)
        bookCoverBytes = BytesIO()
        imageFormat = imageFormat.lower()
        if imageFormat == 'png':
            self.bookCoverFilename = 'cover.png'
        elif imageFormat == 'jpg':
            self.bookCoverFilename = 'cover.jpg'
        defaultCover.save(bookCoverBytes, format=imageFormat)
        self.bookCoverBytes = bookCoverBytes.getvalue()
        return

    def setCover(self, imageBytes, imageFormat: str = 'png'):
        self.bookCoverBytes = imageBytes
        imageFormat = imageFormat.lower()
        if imageFormat == 'png':
            self.bookCoverFilename = 'cover.png'
        elif imageFormat == 'jpg' or imageFormat == 'jpeg':
            self.bookCoverFilename = 'cover.jpg'
        elif imageFormat == 'gif':
            self.bookCoverFilename = 'cover.gif'
        self.hasBookCover = True
    
    def setCoverByPath(self, imagePath: str, imageFormat: str = 'png') -> bool:
        if os.path.exists(imagePath):
            with open (imagePath, 'rb') as file:
                self.setCover(file.read(), imageFormat)
            return True
        return False

    def getCoverBytes(self):
        return self.bookCoverBytes

    def createNewChapter(self, chapterTitle: str, chapterId: str, chapterContent: str):
        chapterFilename = f"xhtml-{chapterId}.xhtml"
        obj = epub.EpubHtml(title=chapterTitle, file_name=chapterFilename, lang=self.bookLanguage, content=chapterContent)
        return (obj, chapterFilename, chapterId)

    def build(self):
        # create epub object
        book = epub.EpubBook()
        book.set_identifier(self.bookId)
        book.set_title(self.bookTitle)
        book.set_language(self.bookLanguage)
        bookSpine = ["nav"]
        bookTableOfContent = []

        if self.bookAuthor:
            for author in self.bookAuthor:
                book.add_author(author)

        # create a cover
        if self.bookCoverBytes != None:
            book.set_cover(self.bookCoverFilename, self.bookCoverBytes)

        xhtmlResource = []
        if self.xhtmlResource and self.xhtmlResource['style']:
            for cssFileName, cssContent in self.xhtmlResource['style'].items():
                cssObj = epub.EpubItem(uid=self.getObjectId(f'css'), file_name=cssFileName, content=cssContent, media_type='text/css')
                book.add_item(cssObj)
                xhtmlResource.append(cssObj)

        if self.tableOfContent:
            # import images
            for index, item in enumerate(self.tableOfContent):
                if 'type' not in item:
                    continue

                if item['type'] == 'article':
                    pass
                elif item['type'] == 'imageDir':
                    if 'imageDir' not in item:
                        continue
                    if 'name' not in item:
                        continue
                    images = common.getImagesFrom(item['imageDir'])
                    totalImages = len(images)
                    pickFirstPage = False
                    for imageIndex, imageInfo in enumerate(images):
                        imagePath = imageInfo[0]
                        exname = imageInfo[1].strip().strip('.').lower()
                        filename = imageInfo[2] 
                        mediaType = 'image/png'
                        if exname == 'jpg':
                            mediaType = 'image/jpeg'
                        elif exname == 'gif':
                            mediaType = 'image/gif'
    
                        with open(imagePath, 'rb') as f:
                            imageSize = (0, 0)
                            imageData = f.read()
                            imageUid = self.getObjectId(f'img-{index + 1}')
                            imageOutputPath = f"{imageUid}.{exname}"
                            if exname in self.imageFormatConversionTable:
                                imageSetup = self.imageFormatConversionTable[exname]
                                imageData, imageSize = common.convertImageQuality(imageData, imageSetup['to'], imageSetup['quality'])
                                imageOutputPath = f"{imageUid}.{imageSetup['to']}"
                            else:
                                imageSize = common.getImageSize(imageData)
                            imageObj = epub.EpubImage(uid=imageUid, file_name=imageOutputPath, media_type=mediaType, content=imageData)
                            if not self.hasBookCover and self.pickFirstImageToBeBookCover:
                                self.setCover(imageData, common.getImageBytesFormat(imageData))
                                if self.bookCoverBytes != None:
                                    book.set_cover(self.bookCoverFilename, self.bookCoverBytes)

                            book.add_item(imageObj)
                            chapterUid = self.getObjectId(f'{index + 1}')
                            chapterObj, chapterFilename, chapterId = self.createNewChapter(
                                f'{imageIndex+1:04}/{totalImages:04} {(imageIndex + 1) * 100/totalImages:.2f}% - {item["name"]}',
                                chapterUid,
                                self.xhtmlTemplate["image"].format(
                                    imageName=imageOutputPath, 
                                    imageWidth=imageSize[0],
                                    imageHeight=imageSize[1],
                                    imageTitle=imageOutputPath,
                                    imagePath=imageOutputPath
                                )
                            )
                            for itemResource in xhtmlResource:
                                chapterObj.add_item(itemResource)

                            #print(chapterObj.get_body_content())
                            book.add_item(chapterObj)
                            bookSpine.append(chapterObj)

                            if pickFirstPage:
                                pickFirstPage = False
                                bookTableOfContent.append( chapterObj )


        book.spine = bookSpine
        book.toc = tuple(bookTableOfContent)
        book.add_item(epub.EpubNcx())
        book.add_item(epub.EpubNav())
        epub.write_epub(self.outputEPUBPath, book, {})

        return self.outputEPUBPath




from setuptools import setup, find_packages
import os
import re
import pathlib

CWD = pathlib.Path(__file__).parent.resolve()

VERSION = '1.0.0' 
with open(os.path.join(CWD, 'epub_image_helper', '__init__.py'), 'r') as f:
    re_version_pattern = re.compile(r"=[\s]*['\"]([0-9\.]+)['\"]")
    for line in f:
        line = line.strip()
        if line.find('version') >= 0:
            extractVersionInfo = re_version_pattern.search(line)
            if extractVersionInfo:
                VERSION = extractVersionInfo.group(1)

PYTHON_REQUIRES = ">=3.10"
URL = "https://github.com/changyy/epub-image-helper"
DOWNLOAD_URL = "https://pypi.org/project/epub-image-helper/"
DESCRIPTION = 'This tool allows you to easily convert specified photos and images into EPUB e-book format, making it accessible for family and friends. It can be used to create monthly or yearly photo collections for children and transform travel photos into e-books.'
LONG_DESCRIPTION = DESCRIPTION
LONG_DESCRIPTION_TYPE = 'text/plain'
try:
    with open(os.path.join(CWD, "README.md"), 'r') as f:
        data = f.read()
        if len(data) > 10:
            LONG_DESCRIPTION = data
            LONG_DESCRIPTION_TYPE = 'text/markdown'
except Exception as e:
    pass


INSTALL_REQUIRES = ['pandas']
try:
    with open(os.path.join(CWD, "requirements.txt"), 'r') as f:
        INSTALL_REQUIRES = [s.strip() for s in f.read().split("\n")]
except Exception as e:
    pass

DEPENDENCY_LINKS = []
try:
    with open(os.path.join(CWD, "deps-requirements.txt"), 'r') as f:
        DEPENDENCY_LINKS = [s.strip() for s in f.read().split("\n")]
except Exception as e:
    pass

setup(
    name="epub-image-helper", 
    version=VERSION,
    author="Yuan-Yi Chang",
    author_email="<changyy.csie@gmail.com>",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESCRIPTION_TYPE,
    packages=find_packages(),
    install_requires=INSTALL_REQUIRES,
    dependency_links=DEPENDENCY_LINKS,
    keywords=['python', 'epub', 'image', 'photo', 'album', 'pdf'],
    python_requires=PYTHON_REQUIRES,
    url=URL,
    download_url=DOWNLOAD_URL,
    entry_points={
        'console_scripts': [
            'epub-image-helper = epub_image_helper.cmd:main',
        ],
    },
    classifiers= [
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS :: MacOS X",
    ]
)



# -*- encoding: utf-8 -*-
import os
import sys
import argparse
import datetime
import subprocess
import json
import uuid
import time

from epub_image_helper import __version__
from epub_image_helper import common
from epub_image_helper import epubImageHelper

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("imageDir", nargs="*", type=str, help="the directory where the image file is located")
    parser.add_argument("--output", type=str, default='/tmp/test.epub', help="output epub path")
    parser.add_argument("--bookConfig", type=str, default='{}', help="a JSON config for EPUB builder. '@-' will read from stdin")
    parser.add_argument("--bookCover", type=str, default='', help="set a cover image for EPUB")
    parser.add_argument("--bookTitle", type=str, default='Unknown', help="set book title")
    parser.add_argument("--bookId", type=str, default=str(uuid.uuid4()), help="set book uuid info")
    parser.add_argument("--bookAuthor", type=str, default='Unknown', help="set book author info")
    parser.add_argument("--pickFirstImageToBeBookCover", action="store_true", default=False, help="Select the first image as the book cover when the 'bookCover' parameter is not specified")
    parser.add_argument("--debug", action="store_true", default=False, help="show bookConfig only")
    parser.add_argument("--epubcheck", action="store_true", default=False, help="test output epub format via epubcheck command")

    args = parser.parse_args()

    startTime = time.time()

    output = { 'status': False, 'input': {}, 'output': { 'path': None, 'size': 0, 'sizeReadable': None, 'imageCount': 0, 'timeCost': []} , 'error': [] , 'version': __version__ }

    output['input']['output'] = args.output
    output['input']['bookConfig'] = args.bookConfig
    output['input']['bookCover'] = args.bookCover
    output['input']['bookId'] = args.bookId
    output['input']['bookTitle'] = args.bookTitle
    output['input']['bookAuthor'] = args.bookAuthor
    output['input']['epubcheck'] = args.epubcheck
    output['input']['imageDir'] = args.imageDir
    output['input']['pickFirstImageToBeBookCover'] = args.pickFirstImageToBeBookCover
    output['input']['debug'] = args.debug
    output['input']['bookTableOfContent'] = []
    output['input']['imageFormatConversionTable'] = {}

    bookConfig = output['input']['bookConfig']
    if output['input']['bookConfig'] == '@-':
        output['input']['stdin'] = ''
        bookConfigJSONObject = None
        for line in sys.stdin:
            output['input']['stdin'] += line
        bookConfig = output['input']['stdin']

    if bookConfig:
        try:
            bookConfigJSONObject = json.loads(bookConfig)
        except json.JSONDecodeError as e:
            output['error'].append(f'input bookConfig json decode error: {e}')
            print(json.dumps(output, indent=4))
            sys.exit(0)

        if bookConfigJSONObject:
            for field in ['bookCover', 'bookId', 'bookAuthor', 'epubcheck', 'bookTableOfContent', 'imageDir', 'pickFirstImageToBeBookCover', 'debug', 'imageFormatConversionTable']:
                if field in bookConfigJSONObject:
                    output['input'][field] = bookConfigJSONObject[field]

            if type(output['input']['imageDir']) is not list:
                output['input']['imageDir'] = [output['input']['imageDir']]

    if not output['input']['bookTableOfContent'] and not output['input']['imageDir']: 
        output['error'].append("If both 'imageDir' and 'bookTableOfContent' are empty, please set at least one of them.")
    else:
        for item in output['input']['imageDir']:
            if not os.path.exists(item):
                output['error'].append(f'imageDir: [{item}] is not found')
                continue
            try:
                with os.scandir(item):
                    pass
                output['input']['bookTableOfContent'].append(
                    {'type': 'imageDir', 'name': os.path.basename(item) or os.path.basename(os.path.dirname(item)), 'imageDir': item}
                )
            except:
                output['error'].append(f'imageDir: [{item}] is not readable')

    if output['input']['imageFormatConversionTable']:
        for fromImageFormat, imageSetup in output['input']['imageFormatConversionTable'].items():
            if 'to' not in imageSetup:
                output['error'].append(f'check imageFormatConversionTable: {imageSetup} no to')
            if 'quality' not in imageSetup:
                output['error'].append(f'check imageFormatConversionTable: {imageSetup} no quality')

    if output['input']['bookCover']:
        if not os.path.exists(output['input']['bookCover']):
            output['error'].append(f"check bookCover failed: {output['input']['bookCover']} not found")

    output['output']['timeCost'].append(time.time() - startTime)

    if len(output['error']) > 0:
        print(json.dumps(output, indent=4))
        sys.exit(0)

    startTime = time.time()

    obj = epubImageHelper.EPUBImageHelper(
        outputEPUBPath=output['input']['output'], 
        imageBookId=output['input']['bookId'],
        imageBookTitle=output['input']['bookTitle'],
        imageBookAuthor=output['input']['bookAuthor'],
        #tableOfContent=output['input']['bookTableOfContent']
        pickFirstImageToBeBookCover=output['input']['pickFirstImageToBeBookCover'], 
    )

    if output['input']['bookTableOfContent']:
        setTOCStatus = obj.setTableOfContent(output['input']['bookTableOfContent'])
        if setTOCStatus['status'] == False:
            output['error'].append(setTOCStatus)
        elif setTOCStatus['totalImage'] == 0:
            output['error'].append('no input images')
            output['error'].append(setTOCStatus)

    output['output']['timeCost'].append(time.time() - startTime)

    if len(output['error']) > 0:
        print(json.dumps(output, indent=4))
        sys.exit(0)

    if output['input']['bookCover'] and os.path.exists(output['input']['bookCover']):
        checkFilenameEx = output['input']['bookCover'].lower()
        imageFormat = 'png'
        if checkFilenameEx.endswith('.jpg') or checkFilenameEx.endswith('.jpeg'):
            imageFormat = 'jpg'
        elif checkFilenameEx.endswith('.gif'):
            imageFormat = 'gif'
        obj.setCoverByPath(output['input']['bookCover'], imageFormat)
    
    if output['input']['imageFormatConversionTable']:
        for fromImageFormat, imageSetup in output['input']['imageFormatConversionTable'].items():
            obj.enableImageFormatConversion(int(imageSetup['quality']), fromImageFormat, imageSetup['to'])

    if output['input']['debug']:
        print(json.dumps(output, indent=4))
        sys.exit(0)

    startTime = time.time()
    output['output']['path'] = obj.build()
    output['output']['timeCost'].append(time.time() - startTime)
    try:
        output['output']['size'] = os.path.getsize(output['output']['path'])
        output['output']['sizeReadable'] = common.getFileSizeReadable(output['output']['size'])
    except Exception as e:
        pass
    output['status'] = output['output']['size'] > 0
    if output['status'] and output['input']['epubcheck']:
        startTime = time.time()
        output['verify'] = { 'status': False, 'command': [ "epubcheck", output['output']['path'], "--json", "-"], "stdout": None, "stderr": None, "error": None}
        try:
            result = subprocess.run(output['verify']['command'], capture_output=True, text=True)
            try:
                output['verify']['stderr'] = result.stderr.strip()
                if len(output['verify']['stderr']) == 0:
                    output['verify']['status'] = True
                output['verify']['stdout'] = json.loads("\n".join(result.stdout.split("\n")[1:]))
            except json.JSONDecodeError as e:
                output['verify']['error'] = f"JSON decoding failed with error: {e}"
        except subprocess.CalledProcessError as e:
            output['verify']['error'] = str(e.output)
        except Exception as e:
            output['verify']['error'] = str(e)
        output['output']['timeCost'].append(time.time() - startTime)
        output['status'] = output['verify']['status']
    print(json.dumps(output, indent=4))

if __name__ == '__main__':
    main()
